"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path = tslib_1.__importStar(require("path"));
var ejs_1 = require("ejs");
var PLUGIN_NAME = 'HTMLsPlugin';
var compile = function (src, params) { return ejs_1.renderFile(src, params, { async: true }); };
function fixPublicPath(publicPath, url) {
    if (publicPath === void 0) { publicPath = './'; }
    if (isRemoteURL(url)) {
        return url;
    }
    var slash = '/';
    if (publicPath.slice(-1) !== slash) {
        publicPath += slash;
    }
    var prefix = publicPath;
    if (prefix.slice(-1) !== slash) {
        prefix += slash;
    }
    return prefix + url;
}
function isRemoteURL(url) {
    return /(http|https):\/\/([\w.]+\/?)\S*/.test(url);
}
function unique(array) {
    return array
        .map(function (a) { return (isRemoteURL(a) ? a : path.normalize(a)); })
        .filter(function (item, index, array) { return array.indexOf(item) === index; });
}
function getSDK(sdks, entryName) {
    var sdk = sdks[entryName] || [];
    if (typeof sdk === 'string') {
        sdk = [sdk];
    }
    if (sdks['*']) {
        sdk = sdk.concat(sdks['*']);
    }
    return sdk.slice(0);
}
function getDefaultConfig(entryName, props) {
    var template = props.template, favicon = props.favicon, htmls = props.htmls;
    var data = htmls[entryName];
    var oo = tslib_1.__assign({ template: template || path.join(__dirname, 'index.ejs'), favicon: favicon || path.join(__dirname, 'favicon.ico'), filename: entryName + ".html", title: entryName }, data);
    oo.template = path.resolve(process.cwd(), oo.template);
    oo.favicon = path.resolve(process.cwd(), oo.favicon);
    return oo;
}
var HTMLsPlugin = (function () {
    function HTMLsPlugin(props) {
        this.props = tslib_1.__assign({ sdks: {}, htmls: {} }, props);
    }
    HTMLsPlugin.prototype.apply = function (compiler) {
        var _this = this;
        compiler.hooks.emit.tapPromise(PLUGIN_NAME, function (compilation) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
            var publicPath, stats, entrypoints;
            var _this = this;
            return tslib_1.__generator(this, function (_a) {
                switch (_a.label) {
                    case 0:
                        publicPath = compilation.outputOptions.publicPath;
                        stats = compilation.getStats().toJson();
                        entrypoints = stats.entrypoints;
                        return [4, Promise.all(Object.keys(entrypoints).map(function (entryName) { return tslib_1.__awaiter(_this, void 0, void 0, function () {
                                var currentAssets, js, css, scripts, html, params, source;
                                return tslib_1.__generator(this, function (_a) {
                                    switch (_a.label) {
                                        case 0:
                                            currentAssets = entrypoints[entryName].assets;
                                            js = currentAssets.filter(function (n) { return path.extname(n) === '.js'; }).map(function (a) { return fixPublicPath(publicPath, a); });
                                            css = currentAssets.filter(function (n) { return path.extname(n) === '.css'; }).map(function (a) { return fixPublicPath(publicPath, a); });
                                            scripts = getSDK(this.props.sdks, entryName).map(function (a) { return fixPublicPath(publicPath, a); });
                                            html = getDefaultConfig(entryName, this.props);
                                            params = {
                                                jsAssets: unique(scripts.concat(js)),
                                                cssAssets: unique(css.concat(html.css || [])),
                                                title: html.title || ''
                                            };
                                            return [4, compile(html.template, params)];
                                        case 1:
                                            source = _a.sent();
                                            compilation.assets[html.filename] = { size: function () { return source.length; }, source: function () { return source; } };
                                            return [2];
                                    }
                                });
                            }); }))];
                    case 1:
                        _a.sent();
                        return [2];
                }
            });
        }); });
    };
    return HTMLsPlugin;
}());
exports.default = HTMLsPlugin;
