"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = tslib_1.__importDefault(require("path"));
var plugins_1 = require("./plugins");
var util_1 = require("./util");
var getStats_1 = tslib_1.__importDefault(require("./getStats"));
process.on('unhandledRejection', function (err) {
    throw err;
});
function buildCore(context) {
    var webpackConfig = context.webpackConfig;
    webpackConfig = Array.isArray(webpackConfig) ? webpackConfig : [webpackConfig];
    var fileOutputPath;
    webpackConfig.forEach(function (config) {
        fileOutputPath = config.output.path;
    });
    var defer = util_1.PromiseDefer();
    function compileDoneHandler(err, stats) {
        if (err) {
            console.error(err);
            defer.reject(err);
            process.on('exit', function () {
                process.exit(1);
            });
        }
        if (!stats) {
            defer.resolve();
            return;
        }
        try {
            var errors = stats.toJson().errors;
            if (errors && errors.length) {
                process.on('exit', function () {
                    process.exit(1);
                });
            }
        }
        catch (e) {
            console.error(e);
            process.on('exit', function () {
                process.exit(1);
            });
        }
        if (!context.watch || stats.hasErrors()) {
            var buildInfo = stats.toString(getStats_1.default(context.verbose));
            if (stats.hasErrors()) {
                console.error(buildInfo);
            }
            else {
                console.log(buildInfo);
                util_1.notifier.notify({
                    title: 'HOLLOW CLI',
                    subtitle: "SUCCESSFULLY",
                    message: "TIME: " + (stats.stats[0].endTime - stats.stats[0].startTime) + "ms",
                    contentImage: path_1.default.join(__dirname, '../assets/success.png'),
                    sound: true
                });
            }
        }
        defer.resolve();
    }
    var compiler = plugins_1.webpack(webpackConfig);
    if (!context.verbose) {
        compiler.hooks.done.tap('mini-css-extract-plugin', function (stats) {
            stats.stats.forEach(function (stat) {
                stat.compilation.children = stat.compilation.children.filter(function (child) { return !child.name.startsWith('mini-css-extract-plugin'); });
            });
        });
    }
    if (context.watch) {
        compiler.watch(context.watch || 200, compileDoneHandler);
    }
    else {
        compiler.run(compileDoneHandler);
    }
    return defer.promise;
}
exports.default = buildCore;
