"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var plugins_1 = require("./plugins");
var util_1 = require("./util");
var webpack_dev_server_1 = tslib_1.__importDefault(require("webpack-dev-server"));
var createLogger_1 = tslib_1.__importDefault(require("webpack-dev-server/lib/utils/createLogger"));
function colorInfo(msg) {
    return "\u001B[1m\u001B[34m" + msg + "\u001B[39m\u001B[22m";
}
function colorError(msg) {
    return "\u001B[1m\u001B[31m" + msg + "\u001B[39m\u001B[22m";
}
process.on('unhandledRejection', function (err) {
    throw err;
});
function startDevServer(context) {
    var webpackConfig = context.webpackConfig;
    var firstWpOpt = Array.isArray(webpackConfig) ? webpackConfig[0] : webpackConfig;
    var options = firstWpOpt.devServer || {};
    var log = createLogger_1.default(options);
    webpack_dev_server_1.default.addDevServerEntrypoints(webpackConfig, options);
    var defer = util_1.PromiseDefer();
    var compiler;
    try {
        compiler = plugins_1.webpack(webpackConfig);
    }
    catch (e) {
        if (e instanceof plugins_1.WebpackOptionsValidationError) {
            log.error(colorError(e.message));
            process.exit(1);
        }
        defer.reject(e);
    }
    var server;
    try {
        server = new webpack_dev_server_1.default(compiler, options, log);
    }
    catch (e) {
        log.error(colorError(e.message));
        process.exit(1);
        defer.reject(e);
    }
    ;
    ['SIGINT', 'SIGTERM'].forEach(function (sig) {
        process.on(sig, function () {
            server.close();
            process.exit();
        });
    });
    server.listen(options.port, options.host, function (err) {
        if (err)
            throw err;
        defer.resolve(server);
        log.info("\nService is running at " + colorInfo(util_1.createDomain(options)));
    });
    return defer.promise;
}
exports.startDevServer = startDevServer;
